<?php
namespace Typo3\Issue78128\Controller;

/***************************************************************
 *
 *  Copyright notice
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * SecurityController
 */
class MainController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {
	
	/**
	 * tableARepository
	 *
	 * @var \Typo3\Issue78128\Domain\Repository\TableARepository
	 * @inject
	 */
	protected $tableARepository = NULL;
	
	/**
	 * tableBRepository
	 *
	 * @var \Typo3\Issue78128\Domain\Repository\TableBRepository
	 * @inject
	 */
	protected $tableBRepository = NULL;

	/**
	 * tableCRepository
	 *
	 * @var \Typo3\Issue78128\Domain\Repository\TableCRepository
	 * @inject
	 */
	protected $tableCRepository = NULL;
	
	/**
	 * action showAction
	 *
	 * @return void
	 */
	public function showAction()
	{
		$entriesA = $this->tableARepository->findAll();
		$entriesB = $this->tableBRepository->findAll();
		$entriesC = $this->tableCRepository->findAll();
		$this->view->assign('entriesA', $entriesA);
		$this->view->assign('entriesB', $entriesB);
		$this->view->assign('entriesC', $entriesC);
	}
	
	/**
	 * action clearAction
	 *
	 * @return void
	 */
	public function clearAction()
	{
		$this->clearTables();
		$this->addFlashMessage('Cleared tables', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		$this->redirect('show');
	}
	
	/**
	 * action prepareDataAction
	 *
	 * @return void
	 */
	public function prepareDataAction()
	{
		$this->clearTables();
		// Create dummy entries
		$entriesC = array();
		for ($i=0 ; $i < 6; $i++) {
			$entryC = new \Typo3\Issue78128\Domain\Model\TableC();
			$entryC->setName('C' . $i);
			$entriesC[] = $entryC;
		}
		$entriesB = array();
		for ($i=0 ; $i < 4; $i++) {
			$entryB = new \Typo3\Issue78128\Domain\Model\TableB();
			$entryB->addC1($entriesC[floor($i/2) + 0]);
			$entryB->addC2($entriesC[floor($i/2) + 1]);
			$entryB->addC2($entriesC[floor($i/2) + 2]);
			$entriesB[] = $entryB;
		}
		for ($i=0 ; $i < 2; $i++) {
			$entryA = new \Typo3\Issue78128\Domain\Model\TableA();
			$entryA->addB($entriesB[$i*2 + 0]);
			$entryA->addB($entriesB[$i*2 + 1]);
			$entryA->addC($entriesC[$i*3 + 0]);
			$entryA->addC($entriesC[$i*3 + 1]);
			$entryA->addC($entriesC[$i*3 + 2]);
			$this->tableARepository->add($entryA);
		}
		$this->addFlashMessage('Initialized tables', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		$this->redirect('show');
	}
	
	/**
	 * action deleteAAction
	 *
	 * @param \Typo3\Issue78128\Domain\Model\TableA $toRemove
	 * @return void
	 */
	public function deleteAAction(\Typo3\Issue78128\Domain\Model\TableA $toRemove)
	{
		$this->tableARepository->remove($toRemove);
		$this->addFlashMessage('Deleted entry ' . $toRemove->getUid() . ' from table A', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		$this->redirect('show');
	}
	
	/**
	 * action deleteBAction
	 *
	 * @param \Typo3\Issue78128\Domain\Model\TableB $toRemove
	 * @return void
	 */
	public function deleteBAction(\Typo3\Issue78128\Domain\Model\TableB $toRemove)
	{
		$this->tableBRepository->remove($toRemove);
		$this->addFlashMessage('Deleted entry ' . $toRemove->getUid() . ' from table B', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		$this->redirect('show');
	}
	
	/**
	 * remove all entries from the tables
	 *
	 * @return void
	 */
	protected function clearTables()
	{
		// Remove all entries
		// tableB and tableC should be emptied as well because of cascade remove
		foreach($this->tableARepository->findAll() as $entryA) {
			$this->tableARepository->remove($entryA);
		}
		$persistenceManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager')->get('TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager');
		$persistenceManager->persistAll();
	}
}